/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTMessage_h__
#define __RTMessage_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTGlobalSignal_h__
#include <RTGlobalSignal.h>
#endif

#ifndef __RTTimespec_h__
#include <RTTimespec.h>
#endif

class  RTActor;
class  RTController;
class  RTDataObject;
struct RTObject_class;
class  RTProtocol;
struct RTTypedValue;

class RTMessage
{
public: // these really should be private
    RTMessage             * next;

#if OTRTSDEBUG != DEBUG_NONE
    int                     _sendPortId;
    int                     _sendPortIndex;
    int                     _sendProbeId;
    RTTimespec              _sendTime;
#endif

public:
    union
    {
	void              * data;
	const void        * const_data;
    };
    short                   signal;
    unsigned char           priority;

protected:
    unsigned char           _flags;
    RTActor               * _receiver;
    RTProtocol            * _port;
    int                     _portIndex;
    const RTObject_class  * _type;

#if RTMESSAGE_PAYLOAD_SIZE != 0
    // space for a small amount of data
    union
    {
	char                _data_area[ RTMESSAGE_PAYLOAD_SIZE ];
	RTAlignment         _alignment;
    };
#endif

    void                    _internData( void );

    enum
    {
	MaskDeferred  = 1,
	MaskDataOwned = 2
    };

public:
    RTS_INLINE void         clear( void );

    RTS_INLINE              RTMessage( void );
			    RTMessage( const RTMessage & );

    RTS_INLINE              ~RTMessage( void );

    RTS_INLINE int          isRTData( void ) const;

    RTS_INLINE void         setDeferred( void );
    RTS_INLINE void         resetDeferred( void );
    RTS_INLINE int          isDeferred( void ) const;

    RTS_INLINE void         setInvalid( void );
    RTS_INLINE int          isValid( void ) const;

    RTS_INLINE void         setSignal( int );
    RTS_INLINE int          getSignal( void ) const;
    const char            * getSignalName( void ) const;

    RTS_INLINE void         setPriority( int );
    RTS_INLINE int          getPriority( void ) const;

    RTS_INLINE void       * getData( void ) const;
    RTS_INLINE void         setData( const void *, const RTObject_class * );
    RTS_INLINE void         setData( const RTDataObject & );
    RTS_INLINE void         internData( void );

    RTS_INLINE const RTObject_class * getType( void ) const;

    RTS_INLINE RTProtocol * sap( void ) const;
    int                     sapId( void ) const;
    RTS_INLINE int          sapIndex0( void ) const; // 0-based
    RTS_INLINE int          sapIndex( void ) const;  // 1-based

    enum DeliverResult
    {
	Accepted
      , Deferred
      , Rejected // signal or type invalid
    };

    DeliverResult           deliver( void );

    RTMessage             & operator=( const RTMessage & );

    // defer current message against its SAP
    int                     defer( void ) const;

    RTS_INLINE void         setDestination( RTActor *, RTProtocol *, int );
    RTS_INLINE RTActor    * receiver( void ) const;
    RTS_INLINE RTController * context( void ) const;

#if RTRUNTIMEBC
    // this replaces msg->signal in pre-6.1 models
    const RTGlobalSignal  & getGlobalSignal( void ) const;

    // reply methods for run-time backwards compatability
    int                     reply( const RTGlobalSignal & _signal,
				   const void           * _data,
				   const RTObject_class * _type ) const;
    RTS_INLINE int          reply( const RTGlobalSignal & ) const;
    RTS_INLINE int          reply( const RTGlobalSignal &,
				   const RTDataObject   & ) const;
    RTS_INLINE int          reply( const RTGlobalSignal &,
				   const RTTypedValue   & ) const;
#endif // RTRUNTIMEBC
};

#if RTS_INLINES
#include <RTMessage.inl>
#endif

#endif // __RTMessage_h__
