/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTPortDescriptor_h__
#define __RTPortDescriptor_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTFieldOffset_h__
#include <RTFieldOffset.h>
#endif

struct RTProtocolDescriptor;

// This structure describes one port of an actor class.
//
// Field            Meaning
// -----            -------
// name             Design name of the port.
// layerName        Default registration name of the port.
// protocol         Protocol of the port.
// offset           Byte offset of the associated object within the
//                  containing class, relative to RTActor.
// replication      The replication factor of the port.
// id               An integer, uniquely identifying this component
//                  within the containing class.
// properties       Distinguishes between the various kinds of ports
//                  and holds the options selected in the model.
//
// An array of these is addressed by an instance of RTActor_class.
// That array must be sorted by id within kind.

struct RTPortDescriptor
{
    const char                 * name;      // design name
    const char                 * layerName; // default registration name
    const RTProtocolDescriptor * protocol;
    RTFieldOffset                offset;
    int                          replication;
    short                        id;
    short                        properties;

    enum
    {
	ShiftKind         = 0
      , ShiftRegistration = 2
      , ShiftNotification = 5
      , ShiftVisibility   = 6
    };

    enum
    {
	// kind
	MaskKind                        = 3 << ShiftKind

      , KindWired                       = 0
      , KindUnwired                     = 1 << ShiftKind
      , KindSpecial                     = 2 << ShiftKind

	// notification
      , MaskNotification                = 1 << ShiftNotification

      , NotificationDisabled            = 0
      , NotificationEnabled             = 1 << ShiftNotification

	// registration
      , MaskRegisterPublished           = 1 << ShiftRegistration
      , MaskRegisterLocked              = 2 << ShiftRegistration
      , MaskRegisterAuto                = 4 << ShiftRegistration
      , MaskRegistration                = 7 << ShiftRegistration

      , RegisterApplicationUnpublished  = 0
      , RegisterApplicationPublished    = 1 << ShiftRegistration
      , RegisterNotPermitted            = 2 << ShiftRegistration
      , RegisterAutoUnlockedUnpublished = 4 << ShiftRegistration
      , RegisterAutoUnlockedPublished   = 5 << ShiftRegistration
      , RegisterAutoLockedUnpublished   = 6 << ShiftRegistration
      , RegisterAutoLockedPublished     = 7 << ShiftRegistration

	// visibility
      , MaskVisibility                  = 1 << ShiftVisibility

      , VisibilityProtected             = 0
      , VisibilityPublic                = 1 << ShiftVisibility
    };
};

#endif // __RTPortDescriptor_h__
