/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTProtocol_h__
#define __RTProtocol_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTPriority_h_
#include <RTPriority.h>
#endif

#ifndef __RTProtocolDescriptor_h__
#include <RTProtocolDescriptor.h>
#endif

class  RTActor;
class  RTActorProbe;
struct RTBindingEnd;
class  RTController;
class  RTDebugger;
struct RTExceptionSignal;
class  RTGlobalSignal;
struct RTInSignal;
class  RTLayerConnector;
class  RTLayerData;
class  RTMessage;
struct RTObject_class;
struct RTOutSignal;
struct RTPortDescriptor;
struct RTSymmetricSignal;
struct RTTypedValue;

class RTProtocol
{
public:
    RTS_INLINE               RTProtocol( void );
			     ~RTProtocol( void );

    RTController           * context( void ) const;
    RTS_INLINE int           getId( void ) const;
    const RTPortDescriptor * getInfo( void ) const;
    RTS_INLINE const char  * getName( void ) const;
    RTS_INLINE RTActor     * getOwner( void ) const;
    RTS_INLINE int           size( void ) const;
    int                      resize( int new_size );

    RTS_INLINE int           registerSAP( const char * );
    RTS_INLINE int           deregisterSAP( void );

    RTS_INLINE int           registerSPP( const char * );
    RTS_INLINE int           deregisterSPP( void );

    const char             * defaultLayerName( void ) const;

    RTS_INLINE int           isRegistered( void ) const;
    const char             * getRegisteredName( void ) const;

#if RTS_COMPATIBLE <= 520
    int                      registerAs( const char * );
#endif
    int                      deregister( void );

    // Is the given replication index (0-based) connected?
    int                      isBoundAt( int index ) const;

    // Is the given replication index (0-based) connected to the actor?
    int                      isIndexTo( int index, RTActor * ) const;

    // Find the smallest replication index (0-based) which is connected to
    // the given actor. The result is -1 if there is no such index or the
    // id is invalid.
    int                      indexTo( RTActor * ) const;

    // Managing the defer queue: These functions match any signal.

    int                      purge  ( void );
    int                      purgeAt( int index );

    int                      recall     ( void );
    int                      recallFront( void );
    int                      recallAt   ( int index, int front = 0 );

    int                      recallAll     ( void );
    int                      recallAllFront( void );
    int                      recallAllAt   ( int index, int front = 0 );

    // Use this method to request notification of the creation and destruction
    // of bindings to this port.  The signals are 'rtBound' and 'rtUnbound'.
    // If called with a non-zero argument the system will send messages to each
    // of the contained end-ports that is currently bound and as ports become
    // bound.  Calling the function with 0 will prevent such messages from
    // being sent, but will not purge any messages already queued.
    // Note that no messages are sent for ports which are bound prior to the
    // call to this function.

    void                     bindingNotification( int on_off );
    RTS_INLINE int           bindingNotificationRequested( void ) const;

    // Use this method to enable or disable type checking by invoke,
    // send and reply. It is initially enabled. The state has no
    // effect if RTTYPECHECK_SEND == RTTYPECHECK_DONT.

    void                     sendTypeCheckEnable( int on_off );
    RTS_INLINE int           sendTypeCheckEnabled( void ) const;

    RTProtocolDescriptor::Status
			     sendCheck( int                    signal,
					const RTObject_class * type ) const;

    // Use this method to enable or disable type checking as each
    // message is received. It is initially enabled. The state has
    // no effect if RTTYPECHECK_RECEIVE == RTTYPECHECK_DONT.

    void                     receiveTypeCheckEnable( int on_off );
    RTS_INLINE int           receiveTypeCheckEnabled( void ) const;

    RTProtocolDescriptor::Status
			     receiveCheck( int                    signal,
					   const RTObject_class * type ) const;

#if RTRUNTIMEBC
    // Methods for run-time backwards compatability

    friend class RTMessage;

    // Used for mapping incoming signals to global signals.
    const RTGlobalSignal &   getGlobalSignal( int ) const;

    // Used for mapping global signals to outgoing signals.
    int                      getLocalSignal( const RTGlobalSignal & ) const;

    // The old send and invoke methods.
    RTS_INLINE int           send( const RTGlobalSignal & signal,
				   const void           * data,
				   const RTObject_class * type,
				   int                    priority = General );

    RTS_INLINE int           send( const RTGlobalSignal & signal,
				   int                    priority = General );

    RTS_INLINE int           send( const RTGlobalSignal & signal,
				   const RTDataObject   & data,
				   int                    priority = General );

    RTS_INLINE int           send( const RTGlobalSignal & signal,
				   const RTTypedValue   & info,
				   int                    priority = General );

    RTS_INLINE int           invoke( RTMessage            * replyBuffer,
				     const RTGlobalSignal & signal,
				     const void           * data,
				     const RTObject_class * type );

    RTS_INLINE int           invoke( RTMessage            * replyBuffer,
				     const RTGlobalSignal & signal );

    RTS_INLINE int           invoke( RTMessage            * replyBuffer,
				     const RTGlobalSignal & signal,
				     const RTDataObject   & data );

    RTS_INLINE int           invoke( RTMessage            * replyBuffer,
				     const RTGlobalSignal & signal,
				     const RTTypedValue   & info );

    // Single port sends and invokes, called by the port classes
    // in the Protocols.
    RTS_INLINE int           sendAt( int                    index,
				     const RTGlobalSignal & signal,
				     const void           * data,
				     const RTObject_class * type,
				     int                    priority );

    RTS_INLINE int           invokeAt( int                    index,
				       RTMessage            * replyBuffer,
				       const RTGlobalSignal & signal,
				       const void           * data,
				       const RTObject_class * type );
#endif // RTRUNTIMEBC

protected:
    enum
    {
	BindingNotification =  1
      , RegisteredAsSAP     =  2
      , SendTypeCheck       =  4
      , ReceiveTypeCheck    =  8
      , Resized             = 16
    };

    void                     init( const RTPortDescriptor * );

    RTS_INLINE int           getFlags( void ) const;
    void                     setFlag( int );
    void                     resetFlag( int );

    void                     bindAt( int, RTBindingEnd &, RTController * );
    int                      peerAt( int, RTBindingEnd & );
    void                     unbindAt( int, RTController * );

    void                     notifyInit( RTController * );
    void                     notifyBoundAt( int, RTController * );

    // Derived classes define additional incoming signals starting
    // with ( rtiLast_{BASE} + 1 ) and will define rtiLast_{CLASS}
    // as its last signal.
    enum
    {
	rtiLast_RTProtocol = 0
    };

private:
    struct End
    {
	enum State
	{
	    Unbound,
	    Bound,
	    BoundXThread
	};

	State           state;
	RTProtocol    * remote;
	int             remoteIndex;
	const int     * signalMap;

	// these are only called after the priority and type are checked

	int             send( RTController         * sender,
			      RTProtocol           * port,
			      int                    index,
			      int                    signal,
			      const void           * data,
			      const RTObject_class * type,
			      int                    prio );

	int             invoke( RTMessage            * replyBuffer,
				RTController         * invoker,
				RTProtocol           * port,
				int                    index,
				int                    signal,
				const void           * data,
				const RTObject_class * type );
    };

    End                    * _ends;
    const RTPortDescriptor * _info;
    int                      _flags;
    RTLayerData            * _layer;

    // unavailable methods
    RTProtocol( const RTProtocol & );
    RTProtocol & operator=( const RTProtocol & );

    friend struct End;
    friend class  RTActor;
    friend class  RTActorProbe;
    friend class  RTDebugger;
    friend class  RTLayerConnector;

    enum CallStyle { Programmatic, Automatic };

    int                      registerSAP( const char *, CallStyle );
    int                      deregisterSAP( CallStyle );

    int                      registerSPP( const char *, CallStyle );
    int                      deregisterSPP( CallStyle );

    // These classes must have access to the methods below.
    friend struct RTExceptionSignal;
    friend struct RTInSignal;
    friend struct RTOutSignal;
    friend struct RTSymmetricSignal;

    // support for "in" signals

    int                      purge      ( int signal );
    int                      purgeAt    ( int signal, int index );

    int                      recall     ( int signal,            int front );
    int                      recallAt   ( int signal, int index, int front );

    int                      recallAll  ( int signal,            int front );
    int                      recallAllAt( int signal, int index, int front );

protected:
    // support for "out" signals
    int                      invoke( RTMessage            * replyBuffer,
				     int                    signal,
				     const void           * data,
				     const RTObject_class * type );

    int                      invokeAt( int                    index,
				       RTMessage            * replyBuffer,
				       int                    signal,
				       const void           * data,
				       const RTObject_class * type );

    int                      reply( int                    signal,
				    const void           * data,
				    const RTObject_class * type );

    int                      send( int                    signal,
				   const void           * data,
				   const RTObject_class * type,
				   int                    priority );

    int                      sendAt( int                    index,
				     int                    signal,
				     const void           * data,
				     const RTObject_class * type,
				     int                    priority );

private:
    // support for "exception" signals
    int                      raise( int                    signal,
				    const void           * data,
				    const RTObject_class * type );
};

#if RTS_INLINES
#include <RTProtocol.inl>
#endif

#endif // __RTProtocol_h__
