/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTAsciiDecoding.h>
#include <RTFloat.h>

#include <float.h>

int RTAsciiDecoding::get_double( double & value )
{
    RTFloat result;
    int     sign        = 1;
    int     num_digits  = 0;
    bool    got_decimal = false;
    char    byte;

    if( getNonWhite( byte ) == 0 )
	return 0;

    if( byte == '-' )
    {
	sign = -1;
	if( getByte( byte ) == 0 )
	    return 0;
    }
    else if( byte == '+' )
    {
	if( getByte( byte ) == 0 )
	    return 0;
    }

    if( byte == 'I' ) // look for "Inf"
    {
	if( getByte( byte ) == 0 || byte != 'n' )
	    return 0;
	if( getByte( byte ) == 0 || byte != 'f' )
	    return 0;

	static double infinite = DBL_MAX;
	static bool   ready    = false;

	if( ! ready )
	{
	    infinite *= 2.0;
	    ready     = true;
	}

	if( sign < 0 )
	    value = - infinite;
	else
	    value =   infinite;

	return 1;
    }

    if( byte == 'N' ) // look for "NaN"
    {
	if( getByte( byte ) == 0 || byte != 'a' )
	    return 0;
	if( getByte( byte ) == 0 || byte != 'N' )
	    return 0;

	value = 0.0;

	return 1;
    }

    result.clear();

    for(;;)
    {
	if( '0' <= byte && byte <= '9' )
	{
	    ++num_digits;
	    result.mul10( byte - '0' );

	    if( got_decimal )
		result.exp10 -= 1;

	    if( getByte( byte ) == 0 )
		break;
	}
	else if( byte == '.' )
	{
	    if( got_decimal )
	    {
		ungetByte( byte );
		break;
	    }
	    got_decimal = true;
	    if( getByte( byte ) == 0 )
		break;
	}
	else
	{
	    ungetByte( byte );
	    break;
	}
    }

    if( num_digits == 0 )
	return 0;

    // look for exponent
    if( getByte( byte ) != 0 )
    {
	if( ! ( byte == 'e' || byte == 'E' || byte == 'd' || byte == 'D' ) )
	    ungetByte( byte );
	else
	{
	    if( getByte( byte ) == 0 )
		return 0;

	    int exp_sign  = 1;
	    int exp_value = 0;

	    if( byte == '-' )
	    {
		exp_sign = -1;
		if( getByte( byte ) == 0 )
		    return 0;
	    }
	    else if( byte == '+' )
	    {
		if( getByte( byte ) == 0 )
		    return 0;
	    }

	    for( num_digits = 0; ; )
	    {
		if( '0' <= byte && byte <= '9' )
		{
		    exp_value  *= 10;
		    exp_value  += byte - '0';
		    num_digits += 1;

		    if( getByte( byte ) == 0 )
			break;
		}
		else
		{
		    ungetByte( byte );
		    break;
		}
	    }

	    if( num_digits == 0 )
		return 0;

	    if( exp_sign < 0 )
		result.exp10 -= exp_value;
	    else
		result.exp10 += exp_value;
	}
    }

    result.normalize2();

    if( sign < 0 )
	value = - result.value();
    else
	value =   result.value();

    return 1;
}
