/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor_class.h>
#include <RTDaemon.h>
#include <RTDaemonInfo.h>
#include <RTDebugger.h>
#include <RTDiag.h>
#include <RTDiagStream.h>
#include <RTElasticArray.h>
#include <RTEventInfo.h>
#include <RTMemoryUtil.h>
#include <RTPortDescriptor.h>

static int getState( const RTActor_class * info, const char * name )
{
    if( info == (const RTActor_class *)0 || name == (const char *)0 )
	return -1;

    int tempLen = RTMemoryUtil::strlen( name );

    if( tempLen == 0 )
	return -1;

    if( tempLen == 1 && name[0] == ':' ) // ':' is the top state
	return 1;

    // we're not copying the first char (':')
    char * tempName = RTMemoryUtil::strdup( &name[ 1 ], tempLen - 1 );

    int    theParent = 1;
    char * statePtr  = tempName;
    char * slidePtr  = tempName;

    for( ; ; ++slidePtr )
    {
	if( *slidePtr == ':' || *slidePtr == '\0' )
	{
	    char saveChar = *slidePtr;

	    *slidePtr = '\0';

	    for( int s = 0; ; ++s )
	    {
		if( s >= info->numStates )
		{
		    *slidePtr = saveChar;
		    delete [] tempName;
		    return -1;
		}

		if( theParent == info->parentState[ s ] &&
		    RTMemoryUtil::strcmp( statePtr, info->names[ s ] ) == 0 )
		{
		    theParent = s + 1;
		    break;
		}
	    }

	    if( saveChar == '\0' ) // done
	    {
		delete [] tempName;
		return theParent;
	    }

	    *slidePtr++ = saveChar;
	    statePtr    = slidePtr;
	}
    }
}

static int getPortId( const RTActor_class * info, const char * name )
{
    if( info == (const RTActor_class *)0 || name == (const char *)0 )
	return -1;

    const RTPortDescriptor * port = info->ports;

    for( int count = info->numPorts; --count >= 0; ++port )
	if( RTMemoryUtil::strcmp( port->name, name ) == 0 )
	    return port->id;

    return -1;
}

void RTDaemon::setInfo( const RTEventInfo & eventInfo )
{
    for( int iActor = 0; iActor < actorInfo.size(); ++iActor )
    {
	RTDaemonInfo * daemonInfo = (RTDaemonInfo *)actorInfo[ iActor ];

	// continue if no daemonInfo or wrong probe
	if( daemonInfo          == (RTDaemonInfo *)0 ||
	    daemonInfo->probeNr != eventInfo.probeNr )
	{
	    continue;
	}

	if( eventInfo.info != (const RTActor_class *)0 )
	    daemonInfo->info = eventInfo.info;

	// update possible state information
	daemonInfo->stateTarget = getState( daemonInfo->info,
					    daemonInfo->stateStrTarget );

	// update possible port information
	daemonInfo->portId = getPortId( daemonInfo->info,
					daemonInfo->portName );
    }
}
