/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDiag.h>
#include <RTFloat.h>

void RTFloat::assign( double valu )
{
    RTAssert( valu > 0.0 );

    exp_2 = - mantissa_bits;
    exp10 = 0;

    // The first multiplier must be large enough to bring the smallest
    // denormalized value into range. The bit-width of a double should
    // be a sufficient power of two.

    const unsigned big_bits  = 8U * sizeof( double );
    const double   big_scale = pow2( big_bits );

    while( valu < 0.5 )
    {
	exp_2 -= (int)big_bits;
	valu  *= big_scale;
    }

    // now 0.5 <= valu

    const double scale = (double)word_scale; //lint !e922

    while( valu >= scale )
    {
	exp_2 += bits_per_word;
	valu  /= scale;
    }

    // now 0.5 <= valu < scale

    while( valu >= 1.0 )
    {
	exp_2 += 1;
	valu  /= 2.0;
    }

    // now 0.5 <= valu < 1.0

    RTAssert( 0.5 <= valu && valu < 1.0 );

    for( int i = 0; i < mantissa_words; ++i )
    {
	RTAssert( 0.0 <= valu && valu < 1.0 );

	valu *= scale;

	word fragment = (word)valu; //lint !e922

	mantissa[ i ] = fragment;

	valu -= (double)fragment; //lint !e922
    }

    normalize10();
}
