/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTFloat.h>

void RTFloat::multiply( const RTFloat & m )
{
    // The algorithm used here is much like the one you learned in grade
    // school, except that instead of doing one row at a time and adding
    // the rows at the end, we compute one column at a time. This works
    // because we have chosen to leave enough spare bits in each word
    // that the sum of each column will not overflow. The picture below
    // illustrates the structure where the operands are four words each.
    // Note that the result is just the more significant half of the
    // product.
    //
    //                      A  B  C  D
    //                   *  a  b  c  d
    //       =========================
    //       +y +y +y +y | +y +y +y
    //    +              | Ad Bd Cd Dd
    //    +           Ac | Bc Cc Dc
    //    +        Ab Bb | Cb Db
    //    +     Aa Ba Ca | Da
    //       =========== | ===========
    //        W  X  Y  Z |

    dword sum = 0UL;
    word  product[ mantissa_words ];

  {
    // Compute the low-order half of the product. We do not store this
    // part of the result, but we are interested in the carry into the
    // least significant word of the result.

    for( int i = mantissa_words; --i >= 0; )
    {
	for( int j = mantissa_words, k = i; --j >= i; ++k )
	    sum += (dword)mantissa[ j ] * (dword)m.mantissa[ k ];

	sum >>= bits_per_word;
    }
  }

  {
    // Compute the high-order half of the product.

    for( int i = mantissa_words; --i >= 1; )
    {
	for( int j = i, k = 0; --j >= 0; ++k )
	    sum += (dword)mantissa[ j ] * (dword)m.mantissa[ k ];

	product[ i ] = (word)sum;
	sum >>= bits_per_word;
    }
    product[ 0 ] = (word)sum;
  }

    // Adjust exponents and copy new mantissa.
    exp_2 -= m.exp_2;
    exp10 -= m.exp10;
    for( int i = mantissa_words; --i >= 0; )
	mantissa[ i ] = (word)( word_mask & product[ i ] );
}
