/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTController.h>
#include <RTPortDescriptor.h>
#include <RTPriority.h>
#include <RTProtocol.h>
#include <RTProtocolDescriptor.h>

int RTProtocol::send( int                    signal,
		      const void           * data,
		      const RTObject_class * type,
		      int                    priority )
{
    RTController * sender = context();

    sender->trace( RTController::CommSend, this, (const void *)0 );

#if RTTYPECHECK_SEND != RTTYPECHECK_DONT
    switch( sendCheck( signal, type ) )
    {
    case RTProtocolDescriptor::badSignal:
	sender->setError( RTController::badSignal,
			  RTController::CommSend, this, (const void *)0 );
#if RTTYPECHECK_SEND == RTTYPECHECK_FAIL
	return 0;
#else
	break;
#endif

    case RTProtocolDescriptor::badType:
	sender->setError( RTController::badClass,
			  RTController::CommSend, this, (const void *)0 );
#if RTTYPECHECK_SEND == RTTYPECHECK_FAIL
	return 0;
#else
	break;
#endif

    case RTProtocolDescriptor::ok:
    default:
	break;
    }
#endif // RTTYPECHECK_SEND != RTTYPECHECK_DONT

    if( RTPriority::isValid( priority ) == 0 )
    {
	sender->setError( RTController::prio,
			  RTController::CommSend, this, (const void *)0 );
	return 0;
    }

    int   result      = 0;
    int   replication = _info->replication;
    End * end         = _ends;

    for( int index = 0; index < replication; ++index, ++end )
	result += end->send( sender, this, index,
			     signal, data, type, priority );

    return result;
}
