/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDiag.h>
#include <RTRelayPort.h>

int RTRelayPort::reserve( const RTActorRef * home,
			  int                id,
			  int                count,
			  int                size,
			  RTActorRef       * importer )
{
    RTAssert( count > 0 );

    if( importer == (RTActorRef *)0 )
    {
	// This is an incarnate operation: we already
	// verified that the replication is sufficient.
	return 1;
    }
    else
    {
	int indirect = size - directUse( home, id );

	if( indirect < count )
	{
	    // There is no point looking to see if the list of _importers
	    // has enough empty slots: the list is not big enough.
	    return 0;
	}

	RTAssert( _importers != (RTActorRef **)0 );

	for( int i = 0; i < indirect; ++i )
	{
	    if( _importers[ i ] == (RTActorRef *)0 )
	    {
		_importers[ i ] = importer;
		if( --count == 0 )
		    return 1; // we found enough empty slots
	    }
	    else if( _importers[ i ] == importer )
	    {
		// Some ports already reserved by this importer.
		// Erase all reservations made for importer and complain.
		while( --i >= 0 )
		{
		    if( _importers[ i ] == importer )
			_importers[ i ] = (RTActorRef *)0;
		}
		break;
	    }
	}

	return 0;
    }
}
