/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTTimerList_h__
#define __RTTimerList_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTConfig_h__
#include <RTConfig.h>
#endif

class  RTController;
class  RTTimerNode;
struct RTTimespec;

class RTTimerList
{
public:
		     RTTimerList( void );
		     ~RTTimerList( void );

    // How many entries are in the list?
    RTS_INLINE int   length( void ) const;

    // Add node to list. Return true iff new node is at head of list.
    int              insert( RTTimerNode * );

    // Remove a specific entry in the list.
    void             remove( RTTimerNode * );

    // Cancel all entries in the list.
    void             cancelAll( RTController * );

    // Expire all timeouts that should have occured by 'now'.
    void             expire( const RTTimespec & now, RTController * );

    // Fetch the earliest time in the list.
    void             headTime( RTTimespec & ) const;

    class Link
    {
    public:
	RTS_INLINE   Link( void );

    private:
	friend class RTTimerList;

	unsigned     index;
    };

private:
    RTTimerNode   ** nodes;
    unsigned         space;
    unsigned         used;

    // unavailable methods
    RTTimerList( const RTTimerList & );
    RTTimerList & operator=( const RTTimerList & );
};

#if RTS_INLINES
#include <RTTimerList.inl>
#endif

#endif // __RTTimerList_h__
