/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifndef __RTTimerNode_h__
#define __RTTimerNode_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTMessage_h__
#include <RTMessage.h>
#endif

#ifndef __RTTimerId_h__
#include <RTTimerId.h>
#endif

#ifndef __RTTimerList_h__
#include <RTTimerList.h>
#endif

#ifndef __RTTimespec_h__
#include <RTTimespec.h>
#endif

#ifndef __RTTiming_h__
#include <RTTiming.h>
#endif

#if USE_THREADS && ! defined( __RTMutex_h__ )
#include <RTMutex.h>
#endif

class  RTActor;
class  RTController;
struct RTObject_class;
class  RTTimerActor;

class RTTimerNode
{
protected:
    friend class RTController;
    friend class RTTimerActor;
    friend class RTTimerId;
    friend class RTTimerList;

    // creator
    Timing::Base          * sap;

    // doubly-linked chain of nodes created by sap; next used in free list
    RTTimerNode           * prev;
    RTTimerNode           * next;

    // link in pending list
    RTTimerList::Link       link;

    RTController          * owner;
    RTMessage             * msg;
    RTTimespec              expireTime;
    RTTimespec              repeatInterval;
    int                     missedIntervals;
    void                  * data;
    const RTObject_class  * type;

    RTTimerId::Ring         id_ring;

    int                     active; // being delivered?

    // for RTTimerId ring management
    void                    _connect( RTTimerId * );
    static void             _disconnect( RTTimerId * );

    // invalidate all associated RTTimerId objects and disassociated from sap
    void                    _disconnectAll( void );

    void                    buildMsg( RTMessage *, RTActor *, int priority );

    void                    _timeout( RTController * responder );
    void                    _cancel( RTController * );

    RTMessage::DeliverResult _deliver( void );

#if USE_THREADS
    static RTMutex          critical;
#endif

public:
    explicit                RTTimerNode( RTController * originator );
			    ~RTTimerNode( void );

    void                    setValue( RTMessage             *,
				      Timing::Base          *,
				      const RTTimespec      & expireTime,
				      const RTTimespec      & repeatInterval,
				      const void            * _data,
				      const RTObject_class  * _type,
				      int                     _priority );

    void                    clear( void );

    static int              cancel( RTController *, RTTimerId & );

    static
    RTMessage::DeliverResult deliver( const RTMessage * );

private:
    // unavailable methods
    RTTimerNode( const RTTimerNode & );
    RTTimerNode & operator=( const RTTimerNode & );
}; //lint !e1712

#endif // __RTTimerNode_h__
