/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifdef PRAGMA
#pragma implementation "RTThreadInfo.h"
#endif

#include <RTJob.h>
#include <RTSyncObject.h>
#include <RTThread.h>
#include <RTThreadInfo.h>

#include <RTOse.h>

OS_PROCESS( thread_run )
{
    SIGSELECT only_start_sig[] = { 1, RTOseStartSignal };

    // wait for start signal
    SIGNAL * sig = receive( only_start_sig );

    // run thread code using this pointer in start signal
    RTJob        * job  = sig->_start._job;
    RTThreadInfo * info = sig->_start._info;

    free_buf( &sig );

    job->mainLoop();

    info->done = 1;
    info->sync.signal();

    for(;;)
	ose_stop( current_process() );
}

RTThread::RTThread( RTJob * job, int stack_size, int prio )
    : _job (              job )
    , _info( new RTThreadInfo )
{
    _info->proc = create_process( OS_PRI_PROC,
				  (char *)job->name(),
				  thread_run,
				  stack_size,
				  prio,
				  (OSTIME)0,
				  (PROCESS)0,
				  (OS_redir_entry *)0,
				  (OSVECTOR)0,
				  (OSUSER)0 );

    ose_start( _info->proc );

    // send signal containing this pointer
    SIGNAL * sig = alloc( sizeof( RTStartData ), RTOseStartSignal );

    sig->_start._job  = _job;
    sig->_start._info = _info;

    ose_send( &sig, _info->proc );
}

RTThreadInfo::RTThreadInfo( void )
    : sync(            )
    , proc( (PROCESS)0 )
    , done(          0 )
{
}

RTThreadInfo::~RTThreadInfo( void )
{
    while( done == 0 )
	sync.wait();

    kill_proc( proc );
}
